#include <windows.h>
#include <stdio.h>
#include "CON_Console.h"
#include "CON_3D.h"
#include "CON_Bitmap.h"
#include "CON_RetVal.h"
#include "3D_World.h"
#include "3D_Object3D.h"
#include "3D_SpecObject3D.h"
#include "CON_Resource.h"
#include "CON_Texture.h"
#include "CON_Keys.h"

int initConsole(int& Width, int& Height, int& FullScreen, int& NoAccel, Screen* S)
{
  if (S->isModeAvailable(640,480)==0) return -1;
  Width=640;
  Height=480;
  FullScreen=1;
  NoAccel=0;
  setDebugLevel(3);
  enableDiagnostics();
  return 0;
}

// An object subclass that will rotate in place.
class MyShip : public Object3D
{
public:
  MyShip(World3D* W, Mesh* M) : Object3D(W,M,NULL)
  {
    M->setPosition(0.0f,0.0f,20.0f);
  }

  long advance(float Fraction)
  {
    m_Mesh->rotate(0.0f*Fraction,0.0f*Fraction,0.15f*Fraction);
    //m_Mesh->forward(10.0f*Fraction);
    return Object3D::advance(Fraction);
  }
};

// A camera associated object, that will move according to keyboard presses.
class MyViewer : public Viewer
{
public:
  MyViewer(World3D* W, Mesh* M, Camera *Cam, Keyboard& K) : 
    Viewer(W,M,Cam),
    m_K(K)
  {}

  long advance(float Fraction)
  {
    float Rot=1.0f,Adv=100.0f;
    if (m_K[DIK_LEFT])     m_Mesh->rotate(0.0f,0.0f,-Rot*Fraction);
    if (m_K[DIK_RIGHT])    m_Mesh->rotate(0.0f,0.0f, Rot*Fraction);
    if (m_K[DIK_DOWN])     m_Mesh->rotate( Rot*Fraction,0.0f,0.0f);
    if (m_K[DIK_UP])       m_Mesh->rotate(-Rot*Fraction,0.0f,0.0f);
    if (m_K[DIK_ADD])      m_Mesh->forward( Adv*Fraction);
    if (m_K[DIK_SUBTRACT]) m_Mesh->forward(-Adv*Fraction);
    return Viewer::advance(Fraction);
  }

  Keyboard& m_K;
};

// A simple radar to track other objects.
class MyRadar : public Radar
{
public:
  MyRadar(Screen* S, int x, int y, int Width, int Height) :
    Radar(),
    m_Screen(S),
    m_x(x),
    m_y(y),
    m_DrawRadar(newBitmap(Width,Height))
  {
    setRange(500.0f,100);
  }

  ~MyRadar()
  {
    m_DrawRadar->release();
  }

  long render()
  {
    long rc;
    if ((rc=m_DrawRadar->clear())!=0) return rc;
    if ((rc=m_DrawRadar->beginDraw())!=0) return rc;
    for(int i=5;i<m_DrawRadar->getWidth();i+=20)
      m_DrawRadar->line(0,i,m_DrawRadar->getHeight()-1,i);
    for(i=5;i<m_DrawRadar->getHeight();i+=20)
      m_DrawRadar->line(i,0,i,m_DrawRadar->getWidth()-1);
    int x,y,z;
    iterate();
    int Midx=m_DrawRadar->getWidth()/2;
    int Midy=m_DrawRadar->getHeight()/2;
    int ONum=0;
    while (getNext(x,y,z)==0)
    {
      ONum++;
      char buf[20];
      sprintf(buf,"%d",y);
      m_DrawRadar->printXY(Midx+x,Midy-z,0xffffffff,buf);
    }
    m_DrawRadar->endDraw();
    clear();
    return m_Screen->copy(m_DrawRadar,m_x,m_y);
  }

  int     m_x,m_y;
  Screen* m_Screen;
  Bitmap* m_DrawRadar;
};

// An object that will rotate a mesh (loaded as a sphere).
class MyPlanet : public Object3D
{
public:
  MyPlanet(World3D* W, Mesh* M) : Object3D(W,M,NULL) {}

  long advance(float Fraction)
  {
    m_Mesh->rotate(0.0f,Fraction/3,0.0f);
    Object3D::advance(Fraction);
    return 0;
  }
};

int oldcnt=0,curcnt=0;
int Frames=0;
int LastFrame=0;
int FPS=0;
int InitCount=3;
ResourceStream *RS=NULL;
TextureCache   *TC=NULL;
Light          *SL=NULL;
World3D        *W=NULL;
Viewer         *V=NULL;
Mesh          *VO=NULL;


int action(Console* C)
{
  Keyboard& K=*(C->getKeyboard());
  K.update();
  Screen& S=*C->getScreen();

  if (W==NULL)
  {
    // Initialize all global variables.
    RS=newResourceStream();
    TC=newTextureCache(RS);
    W=new World3D(&S,C->getSound());
    if (W->status()!=OK) 
    {
      ReportError(W->status(),"Cannot create world.");
      delete W;
      return -1;
    }
    W->getCamera()->setViewable(200,10,400,300);
    W->getCamera()->setStars(1000);
    {
      VO=newMesh();
      VO->setRadius(5.0f);
      V=new MyViewer(W,VO,W->getCamera(),K);
      W->add(V);
    }
    {
      Vector3D Pos(0.0f,0.0f,-13500.0f);

      Mesh* SunMesh = newBillBoardSphere(Pos.x,Pos.y,Pos.z,3000.0f,230,230,30);
      Object3D* SSun=new Object3D(W,SunMesh,NULL);
      W->add(SSun);


      SL=newSunLight(Pos.x,Pos.y,Pos.z);
      if (SL->status()!=OK) ReportError(SL->status(),"Sun light constructor failed.");
      long rc=W->getCamera()->addLight(SL);
      if (rc!=OK) ReportError(rc,"Add light failed.");
    }


    Radar* R=new MyRadar(&S,10,10,100,100);
    W->setRadar(R);


    {
      void* Texture=TC->load("earth.bmp");
      if (Texture!=NULL)
      {
        Mesh* Earth=newSphere(0.0f,0.0f,7500.0f,4500.0f,Texture);
        long rc=Earth->status();
        if (rc==0) 
        {
          MyPlanet* PEarth=new MyPlanet(W,Earth);
          W->add(PEarth);
        }
        else Earth->release();
      }
    }
  
    {
      Mesh* ShipMesh=newMesh();
      if (0==loadXFile("flat.x",ShipMesh,NULL,0.1f))
      {
        MyShip* ship=new MyShip(W,ShipMesh);
        if (C->getSound()!=NULL)
        {
          SoundClip* SC=loadWaveFile("hum.wav",Sound3D);
          if (SC!=NULL)
          {
            SC->set3DMode(1);
            ship->startSound(SC);
          }
        }
        W->add(ship);
      }
      else ShipMesh->release();
    }

  }

  S.clear(S.getRGB16Color(50,50,50));
  W->render();
  W->advance(getFrameTime());
  
  Frames++;
  curcnt=GetTickCount();
  if (oldcnt==0) oldcnt=curcnt;
  else
  {
    if ((curcnt-oldcnt)>1000) 
    { 
      FPS=(Frames-LastFrame);
      LastFrame=Frames;
      oldcnt+=1000;
    }
  }

  char buf[512];
  S.beginDraw();
  sprintf(buf,"FPS: %d",FPS);
  S.printXY(10,450,-1,buf);
  S.printXY(10,350,-1,"Use Arrows, +- to control camera position.    ESC to exit");
  S.endDraw();
  S.flip();
  if (K[DIK_ESCAPE])
  {
    ReportInfo("Exiting");
    SL->release();
    if (TC!=NULL) TC->release();
    if (RS!=NULL) RS->release();
    delete W;
    return 1; // Terminate application
  }
  return 0;
}