/** \file 2D_Background.h
    2D Tiles & Map classes.

Copyright (c) 1998-1999 by Amir Geva.
This file is part of the Photon Game Development library,
beta release version 0.21.  
Permission is granted to use and copy this file for non-commercial use only.  
Please contact the author concerning commercial usage. 
Amir Geva makes no representations about the suitability of this software for any purpose.
It is provided "as is" without express or implied warranty.

*/
#ifndef H_2D_BACKGROUND
#define H_2D_BACKGROUND

#include "2D_Types.h"
#include "CON_Math2D.h"
#include "CON_Bitmap.h"

typedef
   SpriteSelector Tile;

typedef Tile* TileP;


class TileMap
{
public:
   TileMap(Vector2D size, Vector2D tileSize);
           // size: number of tiles horizontally and vertically
           // tileSize: horizontal and vertical size of each tile
   ~TileMap();

   Vector2D getSize()         { return m_size; }
   Vector2D getTileSize()     { return m_tileSize; }

// draw the section of the map with upper-left corner pos to
// the view within canvas.
   long render(Vector2D& pos, Drawable& canvas, Rect2D& view);

// access grants
   TileP& operator()(int row, int col)             { return m_tiles[row * m_size.y + col]; }
   const TileP& operator()(int row, int col) const { return m_tiles[row * m_size.y + col]; }

private:
   Vector2D    m_size;
   Vector2D    m_tileSize;
   TileP*      m_tiles;
protected: 
   int         m_transparent;
};

typedef
   TileMap BackgroundTileMap;


class ForegroundTileMap : public TileMap
{
   ForegroundTileMap(Vector2D size, Vector2D tileSize);
};

#endif // H_2D_BACKGROUND