/** \file 2D_View.h
    A partial view of an infinite 2D World.

Copyright (c) 1998-1999 by Amir Geva.
This file is part of the Photon Game Development library,
beta release version 0.21.  
Permission is granted to use and copy this file for non-commercial use only.  
Please contact the author concerning commercial usage. 
Amir Geva makes no representations about the suitability of this software for any purpose.
It is provided "as is" without express or implied warranty.

*/
#ifndef H_2D_VIEW
#define H_2D_VIEW

#include <CON_Bitmap.h>
#include <CON_Math2D.h>

/** View2D is the visible rectangle in the infinite 2D world.
    A World2D has an active View2D for rendering its sprites.
*/
class View2D
{
public:
  /** Creates a view with the specified rectangle on the Canvas (screen/bitmap) */
  View2D(Drawable* Canvas, int x1, int y1, int x2, int y2);

  /** Changes the default parameters. */
  void setViewParams(Drawable* Canvas, int x1, int y1, int x2, int y2);

  /** Sets the position of the view in the 2D World.
      This specifies the View's top left corner position.
  */
  void setWorldPosition(int x, int y);

  /** Retrieve the View's position. */
  Vector2D& getWorldPosition();

  /** Must be called before rendering, to ensure correct clipping. */
  int beginRendering();

  /** Render a sprite's bitmap in the specified world coordinates. */
  int renderSprite(Bitmap* BM, Vector2D& Pos, Rect2D R=Rect2D());

  /** Must be called after rendering, to ensure correct clipping. */
  int endRendering();

protected:
  Drawable *m_Canvas;
  Vector2D  m_Position;
  Rect2D    m_Window;
};

#endif // H_2D_VIEW