/** \file 2D_World.h
    A 2D World.  Holds sprites and tiles.

Copyright (c) 1998-1999 by Amir Geva.
This file is part of the Photon Game Development library,
beta release version 0.21.  
Permission is granted to use and copy this file for non-commercial use only.  
Please contact the author concerning commercial usage. 
Amir Geva makes no representations about the suitability of this software for any purpose.
It is provided "as is" without express or implied warranty.

*/
#ifndef H_2D_WORLD
#define H_2D_WORLD

#include <WO_World.h>
#include <WO_Collision.h>
#include <CON_Screen.h>
#include <2D_View.h>

/** An implementation of the generic collision detection (libWO).
    Calls the sprites collision detection routines to determine collision.
*/
class CollisionDetection2D : public CollisionDetection
{
public:
  virtual int isCollision(Object* O1, Object* O2);
};

/** An infinite 2D World.  Can contain a hierarchy of 2D objects,
    and has a view to reflect a certain part of it.
*/
class World2D : public World
{
public:
  /** Construct a 2D World. */
  World2D();

  /** Destruct a 2D World. */
  ~World2D();

  // These methods override the ones defined in WO_World.h

  virtual long add(Object* O);

  /** This method is called once before any rendering is done. */
  virtual long preRender();
  
  /** This method is called once per object before it is rendered.
      If it returns a non-zero value, the object will not be rendered. */
  virtual long preObjectRender(Object* O);

  /** This method is called once after all rendering is done. */
  virtual long postRender();

  /** This method is called once per object after it is rendered. */
  virtual long postObjectRender(Object* O);

  /** This method is called once after any object is advanced */
  virtual long postAdvance(float Fraction);

  /** This method is called once per object after it is advanced */
  virtual long postObjectAdvance(Object* O, float Fraction);

  /** Returns the current active view.
      Currently there can be only one view.
      The default view renders to the screen and covers the entire screen.
  */
  View2D& getActiveView() { return *m_View; }

  /** Returns the status of the object.  A non-zero value indicates an error. */
  long status() const { return m_Status; }

protected:
  long                    m_Status;
  View2D                 *m_View;
  CollisionDetection2D    m_ColDet;
};

#endif // H_2D_WORLD