/** \file 3D_World.h
    3D World.

Copyright (c) 1998-1999 by Amir Geva.
This file is part of the Photon Game Development library,
beta release version 0.21.  
Permission is granted to use and copy this file for non-commercial use only.  
Please contact the author concerning commercial usage. 
Amir Geva makes no representations about the suitability of this software for any purpose.
It is provided "as is" without express or implied warranty.

*/
#ifndef H_3D_WORLD
#define H_3D_WORLD

#include <WO_World.h>
#include <CON_Screen.h>
#include <CON_Sound.h>
#include <3D_Radar.h>

/** This class provides maintenance for 3D Objects.
    It moves them, renders them and checks for events, which
    are sent to them. */
class World3D : public World
{
public:
  World3D(Screen* S, Sound* Snd);
  ~World3D();

  // These methods override the ones defined in WO_Object.h
  // See that file for explanation for each one.

  virtual long add(Object* O);
  virtual long preRender();
  virtual long preObjectRender(Object* O);
  virtual long postRender();
  virtual long postObjectRender(Object* O);

  virtual long postAdvance(float Fraction);
  virtual long postObjectAdvance(Object* O, float Fraction);

  Screen3D* getScreen3D();
  Mesh*     createMesh();
  Camera*   getCamera();

  /** This radar will receive notifications of detectable objects
      in the vicinity of the camera.
      Note: R must be dynamically allocated, and will be deleted by World3D */
  long setRadar(Radar* R);
  long status();

protected:
  class World3DImpl* m_Data;
};

#endif // H_3D_WORLD