/** \file CON_Screen.h
    Screen object.

Copyright (c) 1998-1999 by Amir Geva.
This file is part of the Photon Game Development library,
beta release version 0.21.  
Permission is granted to use and copy this file for non-commercial use only.  
Please contact the author concerning commercial usage. 
Amir Geva makes no representations about the suitability of this software for any purpose.
It is provided "as is" without express or implied warranty.

*/
#ifndef H_CON_SCREEN
#define H_CON_SCREEN

#include <CON_Drawable.h>

/** This is an abstract class representing the 2D Screen.
    One of the subclasses:  FullScreen , WindowScreen should be created.
    Console automatically creates this object. */
class Screen : public virtual Drawable
{
public:
  /** Flips the pages at the end of rendering */
  virtual long flip() = 0;

  /** Check if a display mode is available
      Returns a non-zero value if the mode is available. */
  virtual int  isModeAvailable(int Width, int Height) = 0;

  /** Returns the available video memory for bitmaps. */
  virtual int  getFreeVideoMemory() = 0;
};

#endif // H_CON_SCREEN