/** \file DS_Stack.h
    LIFO Data structure.

Copyright (c) 1998-1999 by Amir Geva.
This file is part of the Photon Game Development library,
beta release version 0.21.  
Permission is granted to use and copy this file for non-commercial use only.  
Please contact the author concerning commercial usage. 
Amir Geva makes no representations about the suitability of this software for any purpose.
It is provided "as is" without express or implied warranty.

*/
#ifndef H_DS_STACK
#define H_DS_STACK

#include <DS_Vector.h>

/** A classic LIFO data structure */
class Stack : protected Vector
{
public:
  Stack(unsigned Flags) : Vector(Flags)
  {}
 
  /** Returns non-zero if the stack is empty. */
  DS_BOOL empty()        { return isEmpty(); }

  /** Returns the top element in the stack. */
  Data peek()            { return lastElement(); }

  /** Returns the top element in the stack. */
  Data top()             { return peek(); }

  /** Returns the top element in the stack and remove it.
      Note: pop() returns garbage in a destructive stack. */
  Data pop()             { Data Item=peek(); removeLastElement(); return Item; }

  /** Push an item to the top of the stack. */
  void push(Data Item)   { addElement(Item); }

  /** Search for an item by utilizing the equality function of Containable. */
  Data search(Data Item) { return elementAt(lastIndexOf(Item)); }
};

#endif // H_DS_STACK