/** \file UTIL_Filters.h
    Generic & Specific iostream filters.

Copyright (c) 1998-1999 by Amir Geva.
This file is part of the Photon Game Development library,
beta release version 0.21.  
Permission is granted to use and copy this file for non-commercial use only.  
Please contact the author concerning commercial usage. 
Amir Geva makes no representations about the suitability of this software for any purpose.
It is provided "as is" without express or implied warranty.

*/
#ifndef H_UTIL_FILTERS
#define H_UTIL_FILTERS

#include <fstream.h>
#include <UTIL_GenStreamBuf.h>

//
// All streams in this library are BINARY only.
// This is useful for creating generic filters for compression,
// encryption, DSP, etc.
//

class OutputFilter : public ostream, public WriteMonitor
{
public:
  OutputFilter(GenStreamBuf* sbuf);
  virtual ~OutputFilter();
  virtual void update(int Bytes);
  int getTotalBytes();
protected:
  int m_Bytes;
};

class InputFilter : public istream
{
public:
  InputFilter(GenStreamBuf* sbuf);
  virtual ~InputFilter();
};

///////////////////////////////////////////////////////////
// Block filters
///////////////////////////////////////////////////////////

// Provides a stream for reading a certain area in another stream.
// The block starting position is the stream current position
// Example:  Limit a stream input to the first 500 bytes (Virtual EOF)
class BlockStream : public InputFilter
{
public:
  BlockStream(istream& Input, int Size);
};

// Provides a BlockStream specific to files.
// Adds seeking capabilities.
class FileBlockStream : public InputFilter
{
public:
  FileBlockStream(ifstream& Input, int Size);
};

///////////////////////////////////////////////////////////
// Memory Filters
///////////////////////////////////////////////////////////

// Streams for data which is in memory.
// These are usually terminators of the filter chain.
class MemoryInputStream : public InputFilter
{
public:
  MemoryInputStream(char* Buffer, int Size);
};

class MemoryOutputStream : public OutputFilter
{
public:
  MemoryOutputStream(char* Buffer, int Size);
};

///////////////////////////////////////////////////////////
// Compression filters
///////////////////////////////////////////////////////////
class DeflateStream : public OutputFilter
{
public:
  DeflateStream(ostream& Output);
};

class InflateStream : public InputFilter
{
public:
  InflateStream(istream& Input);
};

///////////////////////////////////////////////////////////
// Encryption filters
///////////////////////////////////////////////////////////
class EncryptionStream : public OutputFilter
{
public:
  EncryptionStream(ostream& Output, char *Password);
};

class DecryptionStream : public InputFilter
{
public:
  DecryptionStream(istream& Input, char *Password);
};

#endif // H_UTIL_FILTERS