/** \file WO_Collision.h
    Generic collision detection control.

Copyright (c) 1998-1999 by Amir Geva.
This file is part of the Photon Game Development library,
beta release version 0.21.  
Permission is granted to use and copy this file for non-commercial use only.  
Please contact the author concerning commercial usage. 
Amir Geva makes no representations about the suitability of this software for any purpose.
It is provided "as is" without express or implied warranty.

*/
#ifndef H_WO_COLLISION
#define H_WO_COLLISION

#include <WO_Object.h>
#include <DS_Vector.h>

/** 
  Collision interaction between two objects.  It is created by the collision
  detection object, and can be retrieved by using the 
  CollisionDetection::getCollision() method.
*/
class Collision : public Serial_Containable
{
public:
  Collision(Object* O1, Object* O2) : m_O1(O1), m_O2(O2) {}
  Object* m_O1;
  Object* m_O2;
};


/** This class provides mechanizm of checking for interactions between objects.
*/
class CollisionDetection
{
public:
  /** Construct a collision detection object */
  CollisionDetection();

  /** Destroys a collision detection object */
  virtual ~CollisionDetection();

  /** Readys the detection mechanizm, by clearing old results. */
  long       clear();

  // Add an object to be checked for collision. */
  long       add(Object* O);

  /** This method performs the actual checks for collisions
      It returns the number of collisions detected. */
  long       check();

  /** After check was run, query for collision by serial number starting
      at 0.  A non NULL value will return a collision object, to be processed.
      NULL means that there are no more collisions. */
  Collision* getCollision(int Number);

  /** This is the only specific method to check actual two objects for collision.
      Implementations for 3D & 2D are separate. */
  virtual int isCollision(Object* O1, Object* O2)=0;

protected:
  Vector          m_Objects;
  Vector          m_Results;
};

#endif // H_WO_COLLISION